@@ -34,6 +34,7 @@
 #include "vinagre-tab.h"
 #include "vinagre-connect.h"
 #include "vinagre-bookmarks.h"
+#include "vinagre-bookmarks-ui.h"
 #include "vinagre-fav.h"
 #include "vinagre-window-private.h"
 #include "vinagre-prefs.h"
@@ -88,6 +89,12 @@ vinagre_cmd_machine_connect (GtkAction     *action,
     }
 }
 
+static void
+vinagre_cmd_free_string_list (gpointer str, gpointer user_data)
+{
+  g_free (str);
+}
+
 void
 vinagre_cmd_machine_open (GtkAction     *action,
 			  VinagreWindow *window)
@@ -98,7 +105,7 @@ vinagre_cmd_machine_open (GtkAction     *action,
   GtkFileFilter     *filter;
   GSList            *files, *l;
   gchar             *uri;
-  gchar             *error;
+  gchar             *error = NULL;
   GSList            *errors = NULL;
 
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
@@ -124,7 +131,7 @@ vinagre_cmd_machine_open (GtkAction     *action,
       for (l = files; l; l = l->next)
         {
 	  uri = (gchar *)l->data;
-	  conn = vinagre_connection_new_from_file (uri, &error);
+	  conn = vinagre_connection_new_from_file (uri, &error, FALSE);
 
 	  if (conn)
 	    {
@@ -135,9 +142,8 @@ vinagre_cmd_machine_open (GtkAction     *action,
 	    }
 	  else
 	    {
-	      errors = g_slist_append (errors, g_strdup (uri));
-	      if (error)
-	        g_free (error);
+	      errors = g_slist_append (errors, g_strdup_printf ("<i>%s</i>: %s", uri, error?error:_("Unknown error")));
+	      g_free (error);
 	    }
 
 	  g_free (uri);
@@ -146,14 +152,17 @@ vinagre_cmd_machine_open (GtkAction     *action,
     }
 
   if (errors)
-    vinagre_utils_show_many_errors (ngettext ("The following file could not be opened:",
-					      "The following files could not be opened:",
-					      g_slist_length (errors)),
-				    errors,
-				    GTK_WINDOW (window));
+    {
+      vinagre_utils_show_many_errors (ngettext ("The following file could not be opened:",
+						 "The following files could not be opened:",
+						 g_slist_length (errors)),
+				      errors,
+				      GTK_WINDOW (window));
+      g_slist_foreach (errors, vinagre_cmd_free_string_list, NULL);
+      g_slist_free (errors);
+    }
 
   gtk_widget_destroy (dialog);
-
 }
 
 void
@@ -191,6 +200,14 @@ vinagre_cmd_machine_quit (GtkAction     *action,
   gtk_widget_destroy (GTK_WIDGET (window));
 }
 
+/* Edit Menu */
+void
+vinagre_cmd_edit_preferences (GtkAction     *action,
+			      VinagreWindow *window)
+{
+  vinagre_prefs_dialog_show (window);
+}
+
 /* View Menu */
 void
 vinagre_cmd_view_show_toolbar	(GtkAction     *action,
@@ -240,6 +257,15 @@ vinagre_cmd_view_fullscreen (GtkAction     *action,
   vinagre_window_toggle_fullscreen (window);
 }
 
+void
+vinagre_cmd_view_original_size (GtkAction     *action,
+				VinagreWindow *window)
+{
+  g_return_if_fail (VINAGRE_IS_WINDOW (window));
+
+  vinagre_tab_original_size (vinagre_window_get_active_tab (window));
+}
+
 void
 vinagre_cmd_view_scaling (GtkAction     *action,
 			  VinagreWindow *window)
@@ -327,15 +353,25 @@ vinagre_cmd_bookmarks_add (GtkAction     *action,
     }
 }
 
+void
+vinagre_cmd_bookmarks_new_folder (GtkAction     *action,
+				  VinagreWindow *window)
+{
+  g_return_if_fail (VINAGRE_IS_WINDOW (window));
+
+  vinagre_bookmarks_new_folder (vinagre_bookmarks_get_default (),
+				GTK_WINDOW (window));
+}
+
 void
 vinagre_cmd_bookmarks_edit (GtkAction     *action,
 			    VinagreWindow *window)
 {
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
-  g_return_if_fail (VINAGRE_IS_CONNECTION (window->priv->fav_conn_selected));
+  g_return_if_fail (VINAGRE_IS_BOOKMARKS_ENTRY (window->priv->fav_entry_selected));
 
   vinagre_bookmarks_edit (vinagre_bookmarks_get_default (),
-                          window->priv->fav_conn_selected,
+                          window->priv->fav_entry_selected,
                           GTK_WINDOW (window));
 }
 
@@ -344,10 +380,10 @@ vinagre_cmd_bookmarks_del (GtkAction     *action,
 			   VinagreWindow *window)
 {
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
-  g_return_if_fail (VINAGRE_IS_CONNECTION (window->priv->fav_conn_selected));
+  g_return_if_fail (VINAGRE_IS_BOOKMARKS_ENTRY (window->priv->fav_entry_selected));
 
   vinagre_bookmarks_del (vinagre_bookmarks_get_default (),
-                         window->priv->fav_conn_selected,
+                         window->priv->fav_entry_selected,
                          GTK_WINDOW (window));
 }
 
@@ -361,7 +397,7 @@ vinagre_cmd_bookmarks_open (GtkAction     *action,
 
   conn = g_object_get_data (G_OBJECT (action), "conn");
   if (!conn)
-    conn = window->priv->fav_conn_selected;
+    conn = vinagre_bookmarks_entry_get_conn (window->priv->fav_entry_selected);
 
   g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
 
@@ -384,4 +420,4 @@ vinagre_cmd_help_about (GtkAction     *action,
   vinagre_utils_help_about (GTK_WINDOW (window));
 }
 
-/* vim: ts=8 */
+/* vim: set ts=8: */
